package scorch;

/*
  Class:  FakeNetwork
  Author: Mikhail Kruk
  Description: all the network interaction with the server; calls the 
  callback methods from the ScorchApplet
*/

import java.io.*;
import java.util.*;

import scorch.utility.*;

public class FakeNetwork implements Network
{
    private ScorchApplet owner;
    private int playerID = 0;
    private int currentTurn;
    private Vector players, playersAlive;
    private Random rand;

    
    public FakeNetwork(ScorchApplet owner) throws Exception
    {
	if( owner == null )
	    throw new Exception("owner is null!");
	
	this.owner = owner;
	players = new Vector();
	playersAlive = new Vector();
	rand = new Random( (new Date()).getTime() );

	Debug.println("Fake network loaded");
    }
    
    public void sendLogin(PlayerProfile profile, String options) 
    {
	Integer playerId = new Integer( playerID );
	players.addElement( playerId );
	
	if( playersAlive.size() > 0 )
	    {
		playersAlive.insertElementAt
		    ( playerId, 
		      Math.abs( rand.nextInt() % playersAlive.size()) );
	    }
	else
	    {
		playersAlive.addElement( playerId );
	    }
	owner.loggedIn( playerID++, profile, 
			false);
    }

    public void sendAIPlayer(String player) 
    {
	Integer playerId = new Integer( playerID );
	players.addElement( playerId );
	
	if( playersAlive.size() > 0 )
	    {
		playersAlive.insertElementAt
		    ( playerId, 
		      Math.abs( rand.nextInt() % playersAlive.size()) );
	    }
	else
	    {
		playersAlive.addElement( playerId );
	    }
	owner.loggedIn( playerID++, new PlayerProfile(player, ""), 
			true);
    }

    public void sendPlayerSettings(int id, PlayerSettings pSettings) 
    {
	owner.receivePlayerOptions(id, pSettings.tankType);
    }
    
    public void sendGameSettings(GameSettings gSettings) 
    {
	owner.receiveGameSettings( gSettings, (new Date()).getTime() );
	currentTurn = Math.abs(rand.nextInt() % playerID);
	owner.makeTurn(currentTurn);
    }

    public void sendEOT(String dt)
    {
	owner.makeTurn( ((Integer)playersAlive.elementAt(currentTurn)).intValue() );
	currentTurn++;
	if( currentTurn >= playersAlive.size() ) currentTurn = 0;
	System.out.println("fakeNetwork: EOT");
    }

    public void sendMassKill()
    {
	owner.receiveMassKill(); // todo
    }

    public void sendTankDead(int id)
    {
	Integer playerId;
	Debug.println("FakeNetwork: sendTankDead");

	Enumeration enumer = playersAlive.elements();
	while( enumer.hasMoreElements() )
	    {
		playerId = (Integer)enumer.nextElement();
		if( playerId.intValue() == id )
		    {
			playersAlive.removeElement( playerId );
			break;
		    }
	    }

	if( playersAlive.size() <= 1 )
	    {
		enumer = players.elements();
		while( enumer.hasMoreElements() )
		    {
			playerId = (Integer)enumer.nextElement();
			if( playersAlive.size() > 0 )
			    {
				playersAlive.insertElementAt
				    ( playerId, 
				      Math.abs( rand.nextInt() % 
						playersAlive.size()) );
			    }
			else
			    {
				playersAlive.addElement( playerId );
			    }
		    }
		owner.receiveEOR();
	    }
    }

    public void sendUseWeapon(int w_id)
    {
	owner.fire( w_id );
	System.out.println("fakeNetwork: fire");
	Debug.printThreads();
    }

    public void sendUseItem(int i_id, int param)
    {
	owner.receiveUseItem( i_id, param );
    }


    /* ------------------------------------------------------------- */
    public void sendMessage(String s)
    {
	Debug.println("FakeNetwork: tried to send "+s);
    }
    public void run() {}
        public void sendNewUser(PlayerProfile profile, String options)
    {
	Debug.println("fakeNetwork: new user");
    }
    public void sendMaxPlayers(int maxPlayers)
    {
	Debug.println("fakeNetwork: max players");
    }
    public void sendShout(String msg, String player, int idx)
    {
	Debug.println("fakeNetwork: shout");
    }
    public void sendUpdate(int id, int power, int angle)
    {
	//Debug.println("fakeNetwork: send update");
    }
    public void sendUpdateUser(PlayerProfile profile, boolean encrypt)
    {
	Debug.println("fakeNetwork: updateuser");
    }
    public void sendTopTen()
    {
	Debug.println("fakeNetwork: top 10");
    }
    public void quit()
    {
	Debug.println("fakeNetwork: quit");
    }
}
